import datetime

try:
    import simplejson as json
except ImportError:
    import json


class JSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.date):
            return obj.isoformat()

        if isinstance(obj, datetime.datetime):
            return obj.isoformat()

        if isinstance(obj, dict):
            return '\n'.join([u'%s: %s' % (k, v) for k, v in obj.iteritems()])

        return json.JSONEncoder.default(self, obj)



from django.utils.translation import ugettext_lazy as _


ERROR_CODES = {
    '01913': [_('Your bank account will not be charged. We invite you to renew your support now.'), ],
    '01119': [_('Your bank account will not be charged. We invite you to renew your support now.'), ],
    '01909': [_('Your bank account will not be charged. We invite you to renew your support now.'), ],
    '01904': [_('Your bank account will not be charged. We invite you to renew your support now.'), ],
    '01902': [_('Your bank account will not be charged. We invite you to renew your support now.'), ],
    '02317': [_('Your bank account will not be charged. We invite you to renew your support now.'), ],
    '01100': [
        _('It appears that your bank can not accept the payment, your account will not be charged. Thank you for trying again in a few hours / days or to contact your bank.'),
        _('There are several possible explanations to this problem: you have exceeded your payment ceiling, your account does not have sufficient funds ...'),
    ],
    '01200': [
        _('It appears that your bank can not accept the payment, your account will not be charged. Thank you for trying again in a few hours / days or to contact your bank.'),
        _('There are several possible explanations to this problem: you have exceeded your payment ceiling, your account does not have sufficient funds ...'),
    ],
    '01116': [
        _('It appears that your bank can not accept the payment, your account will not be charged. Thank you for trying again in a few hours / days or to contact your bank.'),
        _('There are several possible explanations to this problem: you have exceeded your payment ceiling, your account does not have sufficient funds ...'),
    ],
    '01121': [
        _('It appears that your bank can not accept the payment, your account will not be charged. Thank you for trying again in a few hours / days or to contact your bank.'),
        _('There are several possible explanations to this problem: you have exceeded your payment ceiling, your account does not have sufficient funds ...'),
    ],
    '01108': [
        _('It appears that your bank can not accept the payment, your account will not be charged. Thank you for trying again in a few hours / days or to contact your bank.'),
        _('There are several possible explanations to this problem: you have exceeded your payment ceiling, your account does not have sufficient funds ...'),
    ],
    '01120': [
        _('It appears that your bank can not accept the payment, your account will not be charged. Thank you for trying again in a few hours / days or to contact your bank.'),
        _('There are several possible explanations to this problem: you have exceeded your payment ceiling, your account does not have sufficient funds ...'),
    ],
    '01202': [
        _('Unfortunately, we are unable to accept your payment, your bank account will not be charged.'),
        _('If you contributed to projects on Ulule for more than 2500 euros during the calendar year, please contact the team Ulule (support@ulule.com) in order to uncap your user account.'),
    ],
    '04001': [
        _('Unfortunately, we are unable to accept your payment, your bank account will not be charged.'),
        _('If you contributed to projects on Ulule for more than 2500 euros during the calendar year, please contact the team Ulule (support@ulule.com) in order to uncap your user account.'),
    ],
    '04002': [
        _('Unfortunately, we are unable to accept your payment, your bank account will not be charged.'),
        _('If you contributed to projects on Ulule for more than 2500 euros during the calendar year, please contact the team Ulule (support@ulule.com) in order to uncap your user account.'),
    ],
    '01208': [_('Unfortunately, we are unable to accept your payment, your bank account will not be charged.'), ],
    '01125': [_('Unfortunately, we are unable to accept your payment, your bank account will not be charged.'), ],
    '01122': [_('Unfortunately, we are unable to accept your payment, your bank account will not be charged.'), ],
    '01206': [_('Unfortunately, we are unable to accept your payment, your bank account will not be charged.'), ],
    '03022': [
        _('It looks like you have faced difficulties when checking your credit card, your account will not be charged. This step is mandatory (3D-Secure standard) to ensure secured payments.'),
        _('Different ways exist to validate your payment with a 3D-Secure check: code received by SMS, birthday date, etc. We invite you to contact your bank to know the process with your card.'),
    ],
    '01111': [_('It seems that credit card details provided are not correct. Please try again.'), ],
    '01118': [_('It seems that credit card details provided are not correct. Please try again.'), ],
    '01101': [_('It seems that credit card details provided are not correct. Please try again.'), ],
    '01201': [_('It seems that credit card details provided are not correct. Please try again.'), ],
}



import sys
import time

from datetime import datetime, timedelta

from optparse import make_option

from django.core.management.base import BaseCommand
from django.db import models


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--gap',
                    action='store',
                    type='int',
                    dest='gap',
                    default=30,
                    help='Gap bettween two sync'),
        make_option('--range',
                    action='store',
                    type='int',
                    dest='range',
                    default=1000,
                    help='Range between each querysets'),
        make_option('--limit',
                    action='store',
                    type='int',
                    dest='limit',
                    default=None,
                    help='Limit wallets to sync'),
        make_option('--sleep',
                    action='store',
                    type='float',
                    dest='sleep',
                    default=None,
                    help='Sleep between each sync'),
    )

    can_import_settings = True

    def handle(self, *args, **options):
        from leetchi.exceptions import APIError, DecodeError

        from djleetchi.models import Wallet

        qs = Wallet.objects.filter(models.Q(last_synced__lte=datetime.now() - timedelta(minutes=options.get('gap'))) |
                                   models.Q(last_synced__isnull=True)).extra({'has_last_synced': 'CASE WHEN last_synced IS NULL THEN 0 ELSE 1 END'}).order_by('has_last_synced', 'last_synced', 'id')

        limit = options.get('limit') or qs.count()
        sleep = options.get('sleep')

        for offset in range(0, limit, options.get('range')):

            wallets = qs[offset:offset + options.get('range')]

            for wallet in wallets:
                try:
                    last_synced = wallet.last_synced

                    wallet.sync_amount()

                    print u'Sync wallet for user %s (last synced at %s)' % (wallet.user, last_synced)

                    if sleep:
                        time.sleep(sleep)

                except (APIError, DecodeError), e:
                    sys.stdout.write(e)



from django.db import models

from .api import handler


class ResourceField(models.IntegerField):
    def __init__(self, to=None, *args, **kwargs):
        super(ResourceField, self).__init__(*args, **kwargs)

        self.to = to

    def get_attname(self):
        return '%s_id' % self.name

    def contribute_to_class(self, cls, name):
        super(ResourceField, self).contribute_to_class(cls, name)
        setattr(cls, self.name, ReverseSingleRelatedObjectDescriptor(self))


class ReverseSingleRelatedObjectDescriptor(object):
    def __init__(self, field_with_rel):
        self.field = field_with_rel

    def __get__(self, instance, instance_type=None):

        if instance is None:
            return self

        cache_name = self.field.get_cache_name()
        try:
            return getattr(instance, cache_name)
        except AttributeError:
            val = getattr(instance, self.field.attname)
            if val is None:
                # If NULL is an allowed value, return it.
                if self.field.null:
                    return None

            rel_obj = self.field.to.get(val, handler=handler)

            setattr(instance, cache_name, rel_obj)

            return rel_obj

    def __set__(self, instance, value):
        if instance is None:
            raise AttributeError("%s must be accessed via instance" % self._field.name)

        if value is None and self.field.null is False:
            raise ValueError('Cannot assign None: "%s.%s" does not allow null values.' %
                             (instance._meta.object_name, self.field.name))
        elif value is not None and not isinstance(value, self.field.to):
            raise ValueError('Cannot assign "%r": "%s.%s" must be a "%s" instance.' %
                             (value, instance._meta.object_name,
                              self.field.name, self.field.to._meta.object_name))
        val = value.get_pk()

        setattr(instance, self.field.attname, val)

        setattr(instance, self.field.get_cache_name(), value)


try:
    from south.modelsinspector import add_introspection_rules
except ImportError:
    pass
else:
    add_introspection_rules([], ["^djleetchi\.fields\.ResourceField"])



from django.conf.urls import patterns, url
from django.contrib.admin.views.decorators import staff_member_required

from . import views

urlpatterns = patterns(
    '',
    url(r'^wallet/detail/(?P<wallet_id>\d+)/$',
        staff_member_required(views.WalletDetailView.as_view()),
        name='leetchi_wallet_detail'),

    url(r'^contribution/detail/(?P<contribution_id>\d+)/$',
        staff_member_required(views.ContributionDetailView.as_view()),
        name='leetchi_contribution_detail'),

    url(r'^user/detail/(?P<user_id>\d+)/$',
        staff_member_required(views.UserDetailView.as_view()),
        name='leetchi_user_detail'),

    url(r'^refund/detail/(?P<refund_id>\d+)/$',
        staff_member_required(views.RefundDetailView.as_view()),
        name='leetchi_refund_detail'),

    url(r'^transfer/detail/(?P<transfer_id>\d+)/$',
        staff_member_required(views.TransferDetailView.as_view()),
        name='leetchi_transfer_detail'),

    url(r'^transferrefund/detail/(?P<transferrefund_id>\d+)/$',
        staff_member_required(views.TransferRefundDetailView.as_view()),
        name='leetchi_transferrefund_detail'),

    url(r'^withdrawal/detail/(?P<withdrawal_id>\d+)/$',
        staff_member_required(views.WithdrawalDetailView.as_view()),
        name='leetchi_withdrawal_detail'),
)



from django.db import models
from django.contrib.contenttypes.models import ContentType

from .models import TransferRefund, Transfer, Contribution, Refund
from .helpers import queryset_to_dict


def transfer_refund(instance, user):
    contenttype = ContentType.objects.get_for_model(instance)

    transfers = Transfer.objects.filter(object_id=instance.pk, content_type=contenttype)

    transfer_refund_list = []

    if len(transfers):
        for transfer in transfers:
            transfer_refund = TransferRefund()
            transfer_refund.content_object = instance
            transfer_refund.transfer = transfer
            transfer_refund.user = user
            transfer_refund.save(sync=True)

            transfer_refund_list.append(transfer_refund)

        return transfer_refund_list

    return False


def refund(instance, user):
    contenttype = ContentType.objects.get_for_model(instance)

    contributions = Contribution.objects.filter(object_id=instance.pk,
                                                content_type=contenttype,
                                                is_success=True,
                                                is_completed=True)

    contribution_ids = [contribution.pk for contribution in contributions]

    qs = (Refund.objects.filter(contribution__in=contribution_ids)
          .filter(models.Q(is_success=True, is_completed=True) | models.Q(is_success=False, is_completed=False)))

    refunds = queryset_to_dict(qs, key='contribution_id', singular=False)

    refund_list = []

    if len(contributions):
        for contribution in contributions:
            if not contribution.pk in refunds:
                refund = Refund()
                refund.content_object = instance
                refund.contribution = contribution
                refund.user = user
                refund.save(sync=True)

                refund_list.append(transfer_refund)

        return refund_list

    return False



from django.conf import settings


API_PARTNER_ID = getattr(settings, 'LEETCHI_API_PARTNER_ID', 'partnerID')
API_PRIVATE_KEY = getattr(settings, 'LEETCHI_API_PRIVATE_KEY', 'file://path/to/private_key')
API_PRIVATE_KEY_PASSWORD = getattr(settings, 'LEETCHI_API_PRIVATE_KEY_PASSWORD', '$ecret')
API_USE_SANDBOX = getattr(settings, 'LEETCHI_API_USE_SANDBOX', True)
API_HOST = getattr(settings, 'LEETCHI_API_HOST', None)

ALWAYS_SYNC = getattr(settings, 'LEETCHI_ALWAYS_SYNC', True)



from django.db import models
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _
from django.db.models.signals import post_save
from django.utils import timezone as datetime

from dateutil.relativedelta import relativedelta

from .fields import ResourceField
from .helpers import get_payer
from .compat import User, update_fields
from .tasks import sync_resource, sync_amount
from .api import handler

from . import settings

from leetchi import resources
from leetchi.base import DoesNotExist


class ApiModel(models.Model):
    class Meta:
        abstract = True

    @property
    def resource_id(self):
        return getattr(self, '%s_id' % self.Api.resource_field)

    def sync(self, async=False, commit=True):
        if not hasattr(self, 'Api'):
            return False

        if async is False:

            field_name = self.Api.resource_field

            if self.resource_id is not None:
                return False

            parameters = self.request_parameters()

            resource = self._meta.get_field(field_name).to(**parameters)
            resource.save(handler)

            setattr(self, field_name, resource)

            if commit:
                update_fields(self, fields=(field_name, ))

            return True

        self.save(sync=False)

        sync_resource.delay(self.__class__, self.pk)

    def save(self, *args, **kwargs):
        sync = kwargs.pop('sync', settings.ALWAYS_SYNC)

        if sync is True:
            self.sync(commit=False)

        return super(ApiModel, self).save(*args, **kwargs)


class BaseLeetchi(ApiModel):
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    content_type = models.ForeignKey(ContentType)
    creation_date = models.DateTimeField(default=datetime.now)

    class Meta:
        abstract = True

    def get_tag(self):
        return u'%s.%s:%d' % (self.content_type.app_label,
                              self.content_type.model,
                              self.object_id)


class ContributionQuerySet(models.query.QuerySet):
    def success(self):
        return self.filter(is_success=True)

    def refundable(self):
        now = datetime.now()

        qs = self.filter(creation_date__gt=now - relativedelta(months=10))

        qs = qs.filter(models.Q(card_expiration_date__isnull=True) | models.Q(card_expiration_date__gt=now))

        return qs


class ContributionManager(models.Manager):
    def get_query_set(self):
        return ContributionQuerySet(self.model)

    def success(self):
        return self.get_query_set().success()

    def refundable(self):
        return self.get_query_set().refundable()


class Contribution(BaseLeetchi):
    TYPE_PAYLINE = 1
    TYPE_OGONE = 2
    TYPE_CHOICES = (
        (TYPE_PAYLINE, 'Payline'),
        (TYPE_OGONE, 'Ogone'),
    )

    contribution = ResourceField(resources.Contribution, null=True)
    wallet = ResourceField(resources.Wallet)
    amount = models.IntegerField()
    user = models.ForeignKey(User)
    client_fee_amount = models.IntegerField(default=0)
    return_url = models.CharField(null=True, blank=True, max_length=255)
    template_url = models.CharField(null=True, blank=True, max_length=255)
    payment_url = models.CharField(null=True, blank=True, max_length=255)
    is_completed = models.BooleanField(default=False)
    is_success = models.BooleanField(default=False)
    type = models.PositiveSmallIntegerField(choices=TYPE_CHOICES,
                                            default=TYPE_PAYLINE,
                                            verbose_name=_('Type'),
                                            db_index=True)
    card_expiration_date = models.DateField(null=True)
    card_number = models.CharField(max_length=100, null=True)
    culture = models.CharField(max_length=5, null=True)

    objects = ContributionManager()

    class Meta:
        db_table = 'leetchi_contribution'

    class Api:
        resource_field = 'contribution'

    def __init__(self, *args, **kwargs):
        super(Contribution, self).__init__(*args, **kwargs)

        self.target = None

    @property
    def real_amount(self):
        return self.amount / 100.0

    def request_parameters(self):
        user = get_payer(self.user)

        user_id = user.get_pk()

        if self.target:
            user = get_payer(self.target)

        data = {
            'user_id': user_id,
            'amount': self.amount,
            'client_fee_amount': self.client_fee_amount,
            'return_url': self.return_url,
            'wallet_id': self.wallet_id,
            'tag': self.get_tag(),
            'template_url': self.template_url
        }

        if self.type:
            data['type'] = self.get_type_display()

        if self.culture:
            data['culture'] = self.culture

        if self.template_url:
            data['template_url'] = self.template_url

        return data

    def sync_status(self, commit=True):
        contribution = self.contribution

        if contribution.is_success():
            self.is_success = True
            self.is_completed = True

        elif contribution.is_completed and not contribution.is_succeeded:
            self.is_success = False
            self.is_completed = True

        try:
            payment_card = self.contribution.detail_payment_card
        except DoesNotExist:
            pass
        else:
            if payment_card:
                expiration_date = payment_card.expiration_date_converted

                if expiration_date:
                    self.card_expiration_date = expiration_date

                self.card_number = payment_card.number

        if commit:
            self.save()

    def is_error(self):
        return not self.is_success and self.is_completed

    def sync(self, *args, **kwargs):
        result = super(Contribution, self).sync(*args, **kwargs)

        if result:
            self.payment_url = self.contribution.payment_url

            self.sync_status(commit=True)


class Transfer(BaseLeetchi):
    transfer = ResourceField(resources.Transfer, null=True)
    beneficiary_wallet = ResourceField(resources.Wallet)
    payer = models.ForeignKey(User, related_name='payers')
    beneficiary = models.ForeignKey(User, related_name='beneficiaries')
    amount = models.IntegerField()

    class Meta:
        db_table = 'leetchi_transfer'

    class Api:
        resource_field = 'transfer'

    @property
    def user_id(self):
        return self.payer_id

    @property
    def user(self):
        return self.payer

    def request_parameters(self):
        payer = get_payer(self.payer)

        beneficiary = get_payer(self.beneficiary)

        return {
            'payer_id': payer.get_pk(),
            'beneficiary_id': beneficiary.get_pk(),
            'tag': self.get_tag(),
            'amount': self.amount,
            'beneficiary_wallet_id': self.beneficiary_wallet_id
        }


class TransferRefund(BaseLeetchi):
    transfer_refund = ResourceField(resources.TransferRefund, null=True)
    transfer = models.ForeignKey(Transfer)
    user = models.ForeignKey(User)

    class Meta:
        verbose_name = 'transferrefund'
        db_table = 'leetchi_transferrefund'

    class Api:
        resource_field = 'transfer_refund'

    def request_parameters(self):
        user = get_payer(self.user)

        return {
            'user_id': user.get_pk(),
            'transfer_id': self.transfer.transfer_id,
            'tag': self.get_tag()
        }


class Refund(BaseLeetchi):
    user = models.ForeignKey(User)
    refund = ResourceField(resources.Refund, null=True)
    contribution = models.ForeignKey(Contribution)
    is_success = models.BooleanField(default=False)
    is_completed = models.BooleanField(default=False)

    class Api:
        resource_field = 'refund'

    class Meta:
        db_table = 'leetchi_refund'

    def request_parameters(self):
        return {
            'user_id': get_payer(self.user).get_pk(),
            'contribution_id': self.contribution.contribution_id,
            'tag': self.get_tag()
        }

    def get_contribution(self):
        return self.contribution.contribution

    def sync_status(self, commit=True):
        refund = self.refund

        changed = False

        if refund.is_success():
            changed = True
            self.is_success = True
            self.is_completed = True

        elif refund.is_completed and not refund.is_succeeded:
            changed = True
            self.is_success = False
            self.is_completed = True

        if commit and changed:
            update_fields(self, fields=('is_success', 'is_completed',))

    def sync(self, *args, **kwargs):
        result = super(Refund, self).sync(*args, **kwargs)

        if result:
            self.sync_status(commit=False)


class Beneficiary(ApiModel):
    user = models.ForeignKey(User)
    beneficiary = ResourceField(resources.Beneficiary, null=True)
    bank_account_owner_name = models.CharField(max_length=255)
    bank_account_owner_address = models.CharField(max_length=255)
    bank_account_iban = models.CharField(max_length=100)
    bank_account_bic = models.CharField(max_length=100)
    creation_date = models.DateTimeField(default=datetime.now)

    class Meta:
        db_table = 'leetchi_beneficiary'

    class Api:
        resource_field = 'beneficiary'

    def request_parameters(self):
        return {
            'user': get_payer(self.user),
            'bank_account_bic': self.bank_account_bic,
            'bank_account_iban': self.bank_account_iban,
            'bank_account_owner_address': self.bank_account_owner_address,
            'bank_account_owner_name': self.bank_account_owner_name
        }


class Withdrawal(BaseLeetchi):
    amount = models.IntegerField(help_text=_(u'Amount to transfer (in cents, ex: 51900)'),
                                 null=True)
    client_fee_amount = models.IntegerField(help_text=_(u'Amount to transfer with tax (ex: 4152 = 51900 * 8%)'),
                                            null=True)
    beneficiary = models.ForeignKey(Beneficiary, null=True, blank=True)

    withdrawal = ResourceField(resources.Withdrawal, null=True)

    user = models.ForeignKey(User, null=True, blank=True)
    wallet = ResourceField(resources.Wallet, null=True, blank=True)

    is_completed = models.BooleanField(default=False)
    is_succeeded = models.BooleanField(default=False)

    class Api:
        resource_field = 'withdrawal'

    class Meta:
        db_table = 'leetchi_withdrawal'

    @property
    def real_amount(self):
        if not self.amount:
            return 0

        return self.amount / 100.0

    @property
    def is_success(self):
        return self.is_succeeded

    def sync_status(self, commit=True):
        withdrawal = self.withdrawal

        changed = False

        for field_name in ('is_succeeded', 'is_completed', ):
            if getattr(self, field_name) == getattr(withdrawal, field_name):
                continue

            setattr(self, field_name, getattr(withdrawal, field_name))

            changed = True

        if commit and changed:
            update_fields(self, fields=('is_succeeded', 'is_completed', ))

    def request_parameters(self):
        params = {
            'beneficiary_id': self.beneficiary.beneficiary_id,
            'client_fee_amount': self.client_fee_amount or 0,
            'amount': self.amount
        }

        if self.user_id:
            user = get_payer(self.user)

            params['user'] = user

        if self.wallet_id:
            params['wallet'] = self.wallet

        return params


class WalletManager(models.Manager):
    def get_for_model(self, instance):
        try:
            content_type = ContentType.objects.get_for_model(instance)
            return (self.filter(content_type=content_type,
                                object_id=instance.pk)
                    .order_by('creation_date')[0])
        except IndexError:
            return None

    def contribute_to_class(self, cls, name):
        post_save.connect(self.post_save, sender=Contribution)
        post_save.connect(self.post_save, sender=Transfer)
        post_save.connect(self.post_save, sender=TransferRefund)
        post_save.connect(self.post_save, sender=Refund)
        post_save.connect(self.post_save, sender=Withdrawal)
        return super(WalletManager, self).contribute_to_class(cls, name)

    def post_save(self, instance, **kwargs):
        if instance.user_id:
            wallet = self.get_for_model(instance.user)

            if wallet:
                sync_amount.apply_async((wallet.pk, ), countdown=120)


class Wallet(BaseLeetchi):
    user = ResourceField(resources.User, null=True, blank=True)
    wallet = ResourceField(resources.Wallet, null=True, blank=True)
    amount = models.IntegerField(null=True, blank=True)
    last_synced = models.DateTimeField(null=True, blank=True)

    objects = WalletManager()

    class Meta:
        unique_together = (
            ('wallet', 'content_type', 'object_id'),
        )
        db_table = 'leetchi_wallet'

    def sync_amount(self, commit=True, async=False):
        if async is True:
            return sync_amount.delay(self.pk)

        user = self.user

        self.amount = user.personal_wallet_amount
        self.last_synced = datetime.now()

        if commit:
            update_fields(self, fields=('amount', 'last_synced', ))

    @property
    def real_amount(self):
        if not self.amount:
            return 0

        return self.amount / 100.0


class StrongAuthentication(ApiModel):
    strong_authentication = ResourceField(resources.StrongAuthentication, null=True)

    user = models.ForeignKey(User)
    beneficiary = models.ForeignKey(Beneficiary,
                                    related_name='strong_authentication',
                                    null=True, blank=True)

    is_completed = models.BooleanField(default=False)
    is_succeeded = models.BooleanField(default=False)
    creation_date = models.DateTimeField(default=datetime.now)

    class Meta:
        db_table = 'leetchi_strongauthentication'

    class Api:
        resource_field = 'strong_authentication'

    def request_parameters(self):
        beneficiary = None

        if self.beneficiary:
            beneficiary = self.beneficiary.beneficiary_id

        return {
            'user': get_payer(self.user),
            'beneficiary_id': beneficiary
        }


def get_pending_amount(user):
    result = (Withdrawal.objects.filter(user=user, is_completed=False)
              .aggregate(amount=models.Sum('amount')))

    amount = result.get('amount', 0) or 0

    result = (Refund.objects.filter(user=user, is_completed=False)
              .aggregate(amount=models.Sum('contribution__amount')))

    amount += result.get('amount', 0) or 0

    return amount



# -*- coding: utf-8 -*-
from django.views.generic.base import View
from django.http import HttpResponseServerError, Http404

from leetchi import resources
from leetchi.base import DoesNotExist as ResourceDoesNotExist

from .api import handler
from .http import JSONResponse
from .encoders import JSONEncoder


class BaseResourceDetailView(View):
    def get(self, request, *args, **kwargs):

        resource_id = int(kwargs.get(self.resource_id))

        data = {}

        try:
            resource = self.resource_class.get(resource_id, handler)
        except ResourceDoesNotExist:
            raise Http404
        else:
            if resource:

                for attribute in self.attributes:
                    if '__' in attribute:
                        attrs = attribute.split('__')

                        method = resource

                        for attr in attrs:
                            if not hasattr(method, attr):
                                continue

                            method = getattr(method, attr)

                            if attr == attrs[-1]:
                                data[attribute] = method
                    else:
                        data[attribute] = getattr(resource, attribute)

            return JSONResponse(data, cls=JSONEncoder)

        return HttpResponseServerError("Error")


class ContributionDetailView(BaseResourceDetailView):
    resource_class = resources.Contribution
    resource_id = 'contribution_id'
    attributes = ('is_succeeded', 'is_completed', 'creation_date',
                  'detail_payment_card__expiration_date',
                  'detail_payment_card__number',
                  'detail_payment_card__type',
                  'update_date', 'amount', 'id', 'error')


class WalletDetailView(BaseResourceDetailView):
    resource_class = resources.Wallet
    resource_id = 'wallet_id'
    attributes = ('name', 'spent_amount', 'amount',
                  'collected_amount', 'remaining_amount', 'is_closed')


class RefundDetailView(BaseResourceDetailView):
    resource_class = resources.Refund
    resource_id = 'refund_id'
    attributes = ('creation_date', 'update_date', 'user_id',
                  'contribution_id', 'is_succeeded', 'is_completed', 'error')


class TransferDetailView(BaseResourceDetailView):
    resource_class = resources.Transfer
    resource_id = 'transfer_id'
    attributes = ('creation_date', 'update_date', 'payer_id',
                  'beneficiary_id', 'amount', 'payer_wallet_id', 'beneficiary_wallet_id')


class TransferRefundDetailView(BaseResourceDetailView):
    resource_class = resources.TransferRefund
    resource_id = 'transferrefund_id'
    attributes = ('transfer_id', 'user_id', 'creation_date', 'update_date')


class WithdrawalDetailView(BaseResourceDetailView):
    resource_id = 'withdrawal_id'
    resource_class = resources.Withdrawal
    attributes = ('creation_date', 'update_date', 'user_id', 'amount', 'client_fee_amount',
                  'wallet_id', 'is_succeeded', 'is_completed', 'error', 'bank_account_owner_name',
                  'bank_account_owner_address', 'bank_account_iban', 'bank_account_bic')


class UserDetailView(BaseResourceDetailView):
    resource_class = resources.User
    resource_id = 'user_id'
    attributes = ('email', 'id', 'personal_wallet_amount',
                  'birthday', 'nationality')

user_detail = UserDetailView.as_view()
